package com.persagy.dmp.rwd.define.dao;

import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.baomidou.mybatisplus.core.toolkit.Constants;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.persagy.dmp.define.entity.ObjectInfoDefine;
import org.apache.ibatis.annotations.Param;

import java.util.List;
import java.util.Map;

/**
 * 信息点 DAO
 * @author Charlie Yu
 * @date 2021-06-25
 */
public interface ObjectInfoMapper extends BaseMapper<ObjectInfoDefine> {

    /**
     * 按类型查询
     * @param projectIds 不允许为空
     * @param classCodes 参数传入，能大幅提升查询效率
     * @param wrapper 不允许为空
     * @return
     */
    List<ObjectInfoDefine> queryByClassCodes(@Param("projectIds") Iterable<String> projectIds,
                                             @Param("classCodes") Iterable<String> classCodes,
                                             @Param(Constants.WRAPPER) Wrapper<ObjectInfoDefine> wrapper);

    /**
     * 查询采集信息
     * @param projectIds 不允许为空
     * @param classCodes
     * @param wrapper 不允许为空
     * @return
     */
    List<Map<String, Object>> queryCollectInfos(@Param("projectIds") Iterable<String> projectIds,
                                                @Param("classCodes") Iterable<String> classCodes,
                                                @Param(Constants.WRAPPER) Wrapper<ObjectInfoDefine> wrapper);

    /**
     * 查询采集信息
     * @param pageable
     * @param projectIds 不允许为空
     * @param classCodes
     * @param wrapper 不允许为空
     * @return
     */
    Page<Map<String, Object>> queryCollectInfos(Page pageable, @Param("projectIds") Iterable<String> projectIds,
                                                @Param("classCodes") Iterable<String> classCodes,
                                                @Param(Constants.WRAPPER) Wrapper<ObjectInfoDefine> wrapper);

    /**
     * 查询采集信息统计
     * @param projectIds
     * @param classCodes
     * @param wrapper
     * @return
     */
    Map<String, Long> queryCollectReport(@Param("projectIds") Iterable<String> projectIds,
                                         @Param("classCodes") Iterable<String> classCodes,
                                         @Param(Constants.WRAPPER) Wrapper<ObjectInfoDefine> wrapper);
}
