package com.persagy.dmp.rwd.define.service;

import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.persagy.dmp.define.entity.ObjectInfoDefine;

import java.util.List;
import java.util.Map;

/**
 * 信息点 接口
 * @author Charlie Yu
 * @date 2021-06-25
 */
public interface IObjectInfoService {

    /**
     * 通过主键查询
     * @param id
     * @return
     */
    ObjectInfoDefine load(String id);

    /**
     * 通过多主键查询
     * @param idList
     * @return
     */
    List<ObjectInfoDefine> queryByIds(List<String> idList);

    /**
     * 按条件计数
     * @param queryWrapper 查询条件
     * @return 结果数量
     */
    Integer queryCount(Wrapper<ObjectInfoDefine> queryWrapper);

    /**
     * 按条件查询
     * @param queryWrapper 查询条件
     * @return 列表
     */
    List<ObjectInfoDefine> queryByCondition(Wrapper<ObjectInfoDefine> queryWrapper);

    /**
     * 按条件查询
     * @param page 分页条件
     * @param queryWrapper 查询条件
     * @return 分页结果
     */
    Page<ObjectInfoDefine> queryByCondition(Page<ObjectInfoDefine> page, Wrapper<ObjectInfoDefine> queryWrapper);

    /**
     * 新增
     * @param voList 待新增数据
     * @return 新增后数据
     */
    List<ObjectInfoDefine> insert(List<ObjectInfoDefine> voList);

    /**
     * 修改
     * @param voList 待修改数据
     * @return 修改后数据
     */
    List<ObjectInfoDefine> update(List<ObjectInfoDefine> voList);

    /**
     * 删除
     * @param idList 主键
     */
    void delete(List<String> idList);

    /***
     * 根据筛选条件和对象类型编码查询信息点数据
     * @param projectId 项目ID 必传
     * @param classCode 对象类型编码
     * @return
     */
    List<ObjectInfoDefine> queryByClassCode(String projectId, String classCode);

    /***
     * 根据筛选条件和对象类型编码查询信息点数据
     * @param projectId 项目ID 必传
     * @param classCodes 对象类型编码集合 为空查所有
     * @return
     */
    List<ObjectInfoDefine> queryByClassCodes(String projectId, Iterable<String> classCodes);

    /***
     * 根据筛选条件和对象类型编码查询信息点数据
     * @param projectId 项目ID 必传
     * @param classCodes 对象类型编码集合 为空查所有
     * @param wrapper 额外筛选条件
     * @return
     */
    List<ObjectInfoDefine> queryByClassCodes(String projectId, Iterable<String> classCodes, QueryWrapper<ObjectInfoDefine> wrapper);

    /**
     * 根据筛选条件和对象类型编码查询信息点数据
     * @param projectId 项目ID 必传
     * @param classCodes : 对象类型编码集合 为空查所有
     * @return Map<classCode, <infoCode, ObjectInfoDefine>>
     */
    Map<String, Map<String, ObjectInfoDefine>> queryInfoMapByClass(String projectId, Iterable<String> classCodes);

    /**
     * 根据筛选条件和对象类型编码查询信息点数据
     * @param projectId 项目ID 必传
     * @param classCodes : 对象类型编码集合 为空查所有
     * @param wrapper : 额外筛选条件
     * @return Map<classCode, <infoCode, ObjectInfoDefine>>
     */
    Map<String, Map<String, ObjectInfoDefine>> queryInfoMapByClass(String projectId, Iterable<String> classCodes, QueryWrapper<ObjectInfoDefine> wrapper);

    /**
     * 查询对象采集信息统计
     * @param projectId
     * @param wrapper
     * @return
     */
    Map<String, Long> queryCollectReport(String projectId, QueryWrapper<ObjectInfoDefine> wrapper);

    /**
     * 查询对象采集信息
     * @param projectId
     * @param wrapper
     * @return
     */
    List<Map<String, Object>> queryCollectInfos(String projectId, QueryWrapper<ObjectInfoDefine> wrapper);

    /**
     * 分页查询对象采集信息
     * @param pageable
     * @param projectId
     * @param wrapper
     * @return
     */
    Page<Map<String, Object>> queryCollectInfos(Page pageable, String projectId, QueryWrapper<ObjectInfoDefine> wrapper);
}
